/*-----------------------------------------------------------------
  nokia_clock by Jack Botner

  program.c

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  20120430 - program written.
-----------------------------------------------------------------*/
#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "main.h"
#include "n3310_lcd.h"

// Global variables
extern volatile unsigned char	ucHours;
extern volatile unsigned char	ucMinutes;
extern volatile unsigned char	ucSeconds;
extern volatile unsigned char	ucFormat;

extern volatile uint8_t  ucEventFlags;			// system event flags
extern volatile uint8_t  ucStopClock;

// Internal functions

static void set_clock(void);
static void set_time_format(void);

/*-----------------------------------------------------------------
  program_clock()
-----------------------------------------------------------------*/
void program_clock()
{
  char			cCont, cMode, cSel;

  clear_bit( ucEventFlags, EVENT_ENCODER_CW );
  clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
  clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
  cSel = 1;
  
  while (1)
  {
    LCD_clear();
	cCont = 1;
    cMode = ( !cSel ) ? MENU_HIGHLIGHT : MENU_NORMAL;
    LCD_write_string( 1, 1, "Time Format", cMode );
    cMode = ( cSel == 1 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
    LCD_write_string( 1, 2, "Set Clock", cMode );
    cMode = ( cSel == 2 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
    LCD_write_string( 1, 4, "Return", cMode );

    while ( cCont )
    {
      if ( test_bit( ucEventFlags, EVENT_POWER_DOWN ) )
	    return;
      if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
      {
	    LCD_clear();
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
		cCont = 0;
		switch ( cSel )
		{
		  case 0:
		    set_time_format();
			cSel = 1;
		    break;
		  case 1:
		    set_clock();
		    return;
		  case 2:
		    return;
		}					
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CCW ) )
      {
	    if ( cSel == 2 )
		  cSel = 0;
		else
		  ++cSel;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
		break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CW ) )
      {
	    if ( !cSel )
		  cSel = 2;
		else
		  --cSel;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CW );
		break;
      }
	}
  }
}

/*-----------------------------------------------------------------
  set_time_format()
-----------------------------------------------------------------*/
static void set_time_format()
{
  char		cMode;
  
  while (1)
  {
    LCD_clear();
    cMode = ( ucFormat == FORMAT_24HRS ) ? MENU_HIGHLIGHT : MENU_NORMAL;
    LCD_write_string( 1, 1, "24 Hr format", cMode );
    cMode = ( ucFormat == FORMAT_12HRS ) ? MENU_HIGHLIGHT : MENU_NORMAL;
    LCD_write_string( 1, 2, "12 Hr format", cMode );

    while ( 1 )
    {
      if ( test_bit( ucEventFlags, EVENT_POWER_DOWN ) )
	    return;
      if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
      {
	    LCD_clear();
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	    return;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CW ) ||
           test_bit( ucEventFlags, EVENT_ENCODER_CCW ) )
      {
	    if ( ucFormat == FORMAT_24HRS )
		  ucFormat = FORMAT_12HRS;
		else
		  ucFormat = FORMAT_24HRS;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CW );
	    clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
		break;
      }
	}
  }  
}

/*-----------------------------------------------------------------
  set_clock()
-----------------------------------------------------------------*/
static void set_clock()
{
  char		cMode, cPrompt;
  char		cWork[8];

  ucStopClock = 1;		// stop the clock from running
  cPrompt = 1;

  // Set the hours. Must be done in 24 hour format.

  while ( cPrompt )
  {
    LCD_clear();
    LCD_write_string( 1, 1, "Set Hours", MENU_NORMAL );
    sprintf( cWork, "%d", (int) ucHours );
    LCD_write_string_big( 1, 3, cWork, MENU_HIGHLIGHT );

    while ( 1 )
    {
      if ( test_bit( ucEventFlags, EVENT_POWER_DOWN ) )
	  {
		ucStopClock = 0;		// run the ckock
	    return;
	  }	  	
      if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
      {
	    LCD_clear();
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
		cPrompt = 0;
	    break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CW ) )
      {
		if ( ucHours == 23 )
		  ucHours = 0;
		else
		  ++ucHours;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CW );
		break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CCW ) )
      {
		if ( !ucHours )
		  ucHours = 23;
		else
		  --ucHours;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
		break;
      }
	}
  }

  cPrompt = 1;
  while ( cPrompt )
  {
    LCD_clear();
    LCD_write_string( 1, 1, "Set Minutes", MENU_NORMAL );
    sprintf( cWork, "%d", (int) ucMinutes );
    LCD_write_string_big( 1, 3, cWork, MENU_HIGHLIGHT );

    while ( 1 )
    {
      if ( test_bit( ucEventFlags, EVENT_POWER_DOWN ) )
	  {
		ucStopClock = 0;		// run the ckock
	    return;
	  }	  	
      if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
      {
	    LCD_clear();
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
		cPrompt = 0;
	    break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CW ) )
      {
		if ( ucMinutes == 59 )
		    ucMinutes = 0;
		  else
		    ++ucMinutes;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CW );
		break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CCW ) )
      {
		if ( !ucMinutes )
		    ucMinutes = 59;
		  else
		    --ucMinutes;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
		break;
      }
	}
  }

  cPrompt = 1;
  while ( cPrompt )
  {
    LCD_clear();
    LCD_write_string( 1, 1, "Set Seconds", MENU_NORMAL );
    sprintf( cWork, "%d", (int) ucSeconds );
    LCD_write_string_big( 1, 3, cWork, MENU_HIGHLIGHT );

    while ( 1 )
    {
      if ( test_bit( ucEventFlags, EVENT_POWER_DOWN ) )
	  {
		ucStopClock = 0;		// run the ckock
	    return;
	  }	  	
      if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
      {
	    LCD_clear();
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
		cPrompt = 0;
	    break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CW ) )
      {
		if ( ucSeconds == 59 )
		    ucSeconds = 0;
		  else
		    ++ucSeconds;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CW );
		break;
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_CCW ) )
      {
		if ( !ucSeconds )
		    ucSeconds = 59;
		  else
		    --ucSeconds;

	    clear_bit( ucEventFlags, EVENT_ENCODER_CCW );
		break;
      }
	}
  }

  ucStopClock = 0;		// let the clock run now
}
